#!/usr/bin/env python

"""
Simple built-in classes and functions.

Copyright (C) 2005, 2006, 2007 Paul Boddie <paul@boddie.org.uk>

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this library; see the file LICENCE.txt
If not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""

class object:
    def __bool__(self):
        # NOTE: Should really test for __len__ and then resort to False.
        # NOTE: See the reference: 3.3.1 Basic customization
        return False

    def __iadd__(self, other):
        return self.__add__(other)

class bool:
    __atomic__ = object

    def __bool__(self):
        return self

    def __str__(self):
        if self:
            return "True"
        else:
            return "False"

class buffer:
    def __init__(self, size):
        if not isinstance(size, int):
            raise TypeError

    def append(self, s):
        if not isinstance(s, str):
            raise TypeError
        pass

    def __str__(self):
        return str()

class dict:
    def __init__(self, *args):
        for key, value in args:
            self[key] = value

    def __setitem__(self, key, value):
        self.key = key
        self.value = value

    def __getitem__(self, key):
        return self.value

class file:
    def write(self, s):
        pass

class float:
    __atomic__ = object

    def __init__(self, number_or_string=None):
        pass

    def __iadd__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __isub__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __add__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __radd__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __sub__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __rsub__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __mul__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __rmul__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __div__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __rdiv__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __floordiv__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __rfloordiv__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __pow__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __rpow__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __lt__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        elif isinstance(other, float):
            return bool()
        else:
            return NotImplemented

    def __gt__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        elif isinstance(other, float):
            return bool()
        else:
            return NotImplemented

    def __le__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        elif isinstance(other, float):
            return bool()
        else:
            return NotImplemented

    def __ge__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        elif isinstance(other, float):
            return bool()
        else:
            return NotImplemented

    def __eq__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        elif isinstance(other, float):
            return bool()
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        elif isinstance(other, float):
            return bool()
        else:
            return NotImplemented

    def __neg__(self):
        return float()

    def __pos__(self):
        return self

    def __str__(self):
        return str()

    def __bool__(self):
        return self != 0

class int:
    __atomic__ = object

    def __init__(self, number_or_string=None):
        pass

    def __iadd__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __isub__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __add__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __radd__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __sub__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __rsub__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __mul__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __rmul__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __div__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __rdiv__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __floordiv__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __rfloordiv__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __pow__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __and__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __rand__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __or__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __ror__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __xor__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __rxor__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __lt__(self, other):
        if isinstance(other, int):
            return bool()
        else:
            return NotImplemented

    def __gt__(self, other):
        if isinstance(other, int):
            return bool()
        else:
            return NotImplemented

    def __le__(self, other):
        if isinstance(other, int):
            return bool()
        else:
            return NotImplemented

    def __ge__(self, other):
        if isinstance(other, int):
            return bool()
        else:
            return NotImplemented

    def __eq__(self, other):
        if isinstance(other, int):
            return bool()
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, int):
            return bool()
        else:
            return NotImplemented

    def __neg__(self):
        return int()

    def __pos__(self):
        return self

    def __str__(self):
        return str()

    def __bool__(self):
        return self != 0

class list:
    def __init__(self, args=None):
        if args is not None:
            for arg in args:
                self.append(arg)

    def __getitem__(self, index):
        if -len(self) <= index < len(self):
            return self.value
        else:
            raise IndexError, index

    def __setitem__(self, index, value):
        if -len(self) <= index < len(self):
            self.value = value
        else:
            raise IndexError, index

    def __getslice__(self, start, end=None):
        if -len(self) <= start < len(self) and -len(self) <= end < len(self) and start <= end:
            return list(self.value)
        else:
            raise IndexError, index

    def __setslice__(self, start, end, slice):
        if -len(self) <= start < len(self) and -len(self) <= end < len(self) and start <= end:
            for value in slice:
                self.value = value
        else:
            raise IndexError, index

    def append(self, value):
        self.value = value

    def __len__(self):
        return int()

    def __add__(self, other):
        for value in other:
            self.value = value
        return self

    __iadd__ = __add__

    def __str__(self):
        return str()

    def __iter__(self):
        return listiterator(self)

    def __bool__(self):
        return self.__len__() != 0

class listiterator:
    def __init__(self, l):
        self.l = l
        self.index = 0

    def next(self):
        if self.index < len(self.l):
            self.index += 1
            return self.l[self.index]
        else:
            raise StopIteration

    def __bool__(self):
        return bool()

class long:
    __atomic__ = object

    def __init__(self, number_or_string=None):
        pass

    def __iadd__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __isub__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __add__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __radd__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __sub__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __rsub__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __and__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __rand__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __or__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __ror__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __xor__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __rxor__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __lt__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        else:
            return NotImplemented

    def __gt__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        else:
            return NotImplemented

    def __le__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        else:
            return NotImplemented

    def __ge__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        else:
            return NotImplemented

    def __eq__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        else:
            return NotImplemented

    def __neg__(self):
        return long()

    def __pos__(self):
        return self

    def __str__(self):
        return str()

    def __bool__(self):
        return self != 0

class none:
    __atomic__ = object

    def __bool__(self):
        return False

    def __str__(self):
        return "None"

class slice:
    def __init__(self, start_or_end, end=None, step=None):
        if end is None:
            self.start = 0
            self.end = start_or_end
        else:
            self.start = start_or_end
            self.end = end
        self.step = step

class str:
    __atomic__ = object

    def __init__(self, x=None):
        x.__str__()

    def __add__(self, other):
        if isinstance(other, str):
            return str()
        else:
            raise TypeError

    def __radd__(self, other):
        if isinstance(other, str):
            return str()
        else:
            raise TypeError

    def __mod__(self, other):

        "The format operator for strings."

        return str()

    def __lt__(self, other):
        if isinstance(other, str):
            return bool()
        else:
            return NotImplemented

    def __gt__(self, other):
        if isinstance(other, str):
            return bool()
        else:
            return NotImplemented

    def __le__(self, other):
        if isinstance(other, str):
            return bool()
        else:
            return NotImplemented

    def __ge__(self, other):
        if isinstance(other, str):
            return bool()
        else:
            return NotImplemented

    def __eq__(self, other):
        if isinstance(other, str):
            return bool()
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, str):
            return bool()
        else:
            return NotImplemented

    def __len__(self):
        return int()

    def __str__(self):
        return self

    def __bool__(self):
        return self.__len__() != 0

    def join(self, l):
        total = 0
        first = 1
        self_len = self.__len__()
        for i in l:
            if not first:
                total += self_len
            total += len(str(i))
            first = 0
        b = buffer(total)
        first = 1
        for i in l:
            if not first:
                b.append(self)
            b.append(str(i))
            first = 0
        s = str(b)
        return s

class tuple:
    def __init__(self, *args):
        for arg in args:
            self.append(arg)

    def __getitem__(self, index):
        if -len(self) <= index < len(self):
            return self.value
        else:
            raise IndexError, index

    def __setitem__(self, index, value):
        if -len(self) <= index < len(self):
            self.value = value
        else:
            raise IndexError, index

    def __getslice__(self, start, end=None):
        if -len(self) <= start < len(self) and -len(self) <= end < len(self) and start <= end:
            return list(self.value)
        else:
            raise IndexError, index

    def __setslice__(self, start, end, slice):
        if -len(self) <= start < len(self) and -len(self) <= end < len(self) and start <= end:
            for value in slice:
                self.value = value
        else:
            raise IndexError, index

    # NOTE: The append method should be internal at most.

    def append(self, value):
        self.value = value

    def __len__(self):
        return int()

    def __add__(self, other):
        for value in other:
            self.value = value
        return self

    def __str__(self):
        return str()

    def __iter__(self):
        return tupleiterator(self)

    def __bool__(self):
        return self.__len__() != 0

class tupleiterator:
    def __init__(self, l):
        self.l = l
        self.index = 0

    def next(self):
        if self.index < len(self.l):
            self.index += 1
            return self.l[self.index]
        else:
            raise StopIteration

    def __bool__(self):
        return bool()

class xrange:
    def __init__(self, start_or_end, end=None, step=1):
        if end is None:
            self.start = 0
            self.end = start_or_end
        else:
            self.start = start_or_end
            self.end = end
        self.step = step
        self.current = self.start

    def __iter__(self):
        return self

    def next(self):
        if self.current >= self.end:
            raise StopIteration
        current = self.current
        self.current += self.step
        return current

class Exception:
    def __init__(self, *args):
        pass

class AssertionError(Exception):
    __atomic__ = object

class AttributeError(Exception):
    __atomic__ = object

class IndexError(Exception):
    __atomic__ = object

class StopIteration(Exception):
    __atomic__ = object

class TypeError(Exception):
    __atomic__ = object

class NotImplementedType:
    __atomic__ = object

# General functions.

def isinstance(obj, cls):
    return bool()

def issubclass(cls1, cls2):
    return bool()

def len(x):
    return x.__len__()

def max(*l):
    max_so_far = l[0]
    for i in l[1:]:
        if i > max_so_far:
            max_so_far = i
    return max_so_far

def range(start_or_end, end=None, step=None):
    if end is None:
        start = 0
        end = start_or_end
    else:
        start = start_or_end
    if start == end:
        return []
    elif start > end:
        step = step or 1
        i = start
        result = []
        while i < end:
            result.append(i)
            i += 1
        return result
    else:
        step = step or -1
        i = start
        result = []
        while i > end:
            result.append(i)
            i -= 1
        return result

# Special values.

True = bool()
False = bool()
None = none()
stdin = file()
stdout = file()
stderr = file()
NotImplemented = NotImplementedType()

# Special functions. These all operate on references at run-time.

def __is__(a, b):
    return bool()

def __not__(a):
    return bool()

# vim: tabstop=4 expandtab shiftwidth=4
